// Copyright (c) 2012, Ladislav Thon. All rights reserved. Use of this source
// code is governed by a BSD-style license that can be found in the LICENSE file.

library detester;

import 'dart:io';
import 'package:darmatch/matchers.dart';
import 'package:dartlings/ansi_term.dart';

part 'src/detester/exceptions.dart';
part 'src/detester/suite.dart';
part 'src/detester/runner.dart';
part 'src/detester/reporter.dart';

/// Creates a [TestSuite].
configure(String name, List<Function> tests,
          [Function before, Function after, Function beforeSuite, Function afterSuite])
    => new TestSuite(name, tests, before, after, beforeSuite, afterSuite);

/// Configures multiple [TestSuite]s, either created by [configure]
/// or by passing a [List] of test functions.
multi(List suites) => new MultipleTestSuites(suites);

/// Evaluates the [expectation] against the [value] and if it doesn't match,
/// throws [ExpectException] with description of the mismatch.
/// If [message] is provided, it is prepended to the exception message.
void expectThat(value, Matcher expectation, [String message]) {
  bool match = expectation.matches(value);
  if (!match) {
    StringBuffer exceptionMessage = new StringBuffer();
    if (message != null) {
      exceptionMessage.add(message).add("\n");
    }
    if (expectation is HasRichMessage) {
      exceptionMessage.add(expectation.buildMessage(value));
    } else {
      exceptionMessage.add("'$value' must ${expectation.description}");
    }
    throw new ExpectException("$exceptionMessage");
  }
}

/// Evaluates the [assumption] against the [value] and if it doesn't match,
/// throws [AssumeException] with description of the mismatch.
/// If [message] is provided, it is prepended to the exception message.
void assumeThat(value, Matcher assumption, [String message]) {
  bool match = assumption.matches(value);
  if (!match) {
    StringBuffer exceptionMessage = new StringBuffer();
    if (message != null) {
      exceptionMessage.add(message).add("\n");
    }
    if (assumption is HasRichMessage) {
      exceptionMessage.add(assumption.buildMessage(value));
    } else {
      exceptionMessage.add("'$value' was assumed to ${assumption.description}");
    }
    throw new AssumeException("$exceptionMessage");
  }
}

