// Copyright (c) 2012, Ladislav Thon. All rights reserved. Use of this source
// code is governed by a BSD-style license that can be found in the LICENSE file.

class TestRunner {
  run(TestSuite suite);
}

class DefaultRunner implements TestRunner {
  final TestReporter reporter;
  bool shouldStop = false;

  DefaultRunner._new(this.reporter);

  factory DefaultRunner(TestReporter reporter) {
    if (reporter == null) {
      reporter = new NullReporter();
    }
    return new DefaultRunner._new(reporter);
  }

  run(TestSuite suite) {
    reporter.start(suite);

    if (suite.beforeSuite != null) {
      try {
        suite.beforeSuite();
      } catch (e) {
        reporter.fatal(e);
        shouldStop = true;
      }
    }

    try {
      for (final test in suite.tests) {
        if (shouldStop) {
          break;
        }
        _runTest(suite, test);
      }
    } catch (e) {
      reporter.fatal(e);
    }

    if (suite.afterSuite != null) {
      try {
        suite.afterSuite();
      } catch (e) {
        reporter.fatal(e);
      }
    }

    reporter.end();
  }

  _runTest(TestSuite suite, Function test) {
    if (suite.before != null) {
      suite.before();
    }

    try {
      reporter.begin(test);
      test();
      reporter.pass(test);
    } on ExpectException catch (e) {
      reporter.fail(test, e);
    } on AssumeException catch (e) {
      reporter.skip(test, e);
    } catch (e) {
      reporter.error(test, e);
      shouldStop = true;
    }

    if (suite.after != null) {
      suite.after();
    }
  }
}

